note
	description: "[
		Root class describing tests to be run on the {GRAPH}
		and its descendents.
		]"
	author: "Jimmy J Johnson"
	date: "8/19/21"

class
	GRAPH_TESTS

inherit

	EQA_TEST_SET
		redefine
			on_prepare
		end

feature {NONE} -- Initialization

	on_prepare
			-- Set up objects used by the tests
		do
				-- Create the containers for bookkeeping
			create graphs.make (10)
			create nodes.make (10)
			create edges.make (10)
				-- Create empty graphs
			create graph_1
			create graph_2
				-- Create some nodes
			create node_a
			create node_b
			create node_c
			create node_d
			create node_e
			create node_f
			create node_g
			create node_h
			create node_i
			create node_j
			create node_x
			create node_y
			create node_z
				-- Create some edges
			create edge_1
			create edge_2
			create edge_3
			create edge_4
			create edge_5
			create edge_6
			create edge_7
			create edge_8
			create edge_9
			create edge_10
			create edge_11
			create edge_12
			create edge_13
			create edge_14
				-- Add graphs to container for bookkeeping
			graphs.extend ("Graph 1", graph_1)
			graphs.extend ("Graph 2", graph_2)
				-- Add nodes to container for bookkeeping
			nodes.extend ("node A", node_a)
			nodes.extend ("node B", node_b)
			nodes.extend ("node C", node_c)
			nodes.extend ("node D", node_d)
			nodes.extend ("node E", node_e)
			nodes.extend ("node F", node_f)
			nodes.extend ("node G", node_g)
			nodes.extend ("node H", node_h)
			nodes.extend ("node I", node_i)
			nodes.extend ("node J", node_j)
			nodes.extend ("node X", node_x)
			nodes.extend ("node y", node_y)
			nodes.extend ("node Z", node_z)
				-- Add edges to container for bookkeeping
			edges.extend ("edge 1", edge_1)
			edges.extend ("edge 2", edge_2)
			edges.extend ("edge 3", edge_3)
			edges.extend ("edge 4", edge_4)
			edges.extend ("edge 5", edge_5)
			edges.extend ("edge 6", edge_6)
			edges.extend ("edge 7", edge_7)
			edges.extend ("edge 8", edge_8)
			edges.extend ("edge 9", edge_9)
			edges.extend ("edge 10", edge_10)
			edges.extend ("edge 11", edge_11)
			edges.extend ("edge 12", edge_12)
			edges.extend ("edge 13", edge_13)
			edges.extend ("edge 14", edge_14)
		end

feature -- Access

	graph_1: like graph_anchor
			-- A graph to be tested

	graph_2: like graph_anchor
			-- Anotgher graph to be tested

	node_a: like node_anchor
	node_b: like node_anchor
	node_c: like node_anchor
	node_d: like node_anchor
	node_e: like node_anchor
	node_f: like node_anchor
	node_g: like node_anchor
	node_h: like node_anchor
	node_i: like node_anchor
	node_j: like node_anchor
	node_x: like node_anchor
	node_y: like node_anchor
	node_z: like node_anchor

	edge_1: like edge_anchor
	edge_2: like edge_anchor
	edge_3: like edge_anchor
	edge_4: like edge_anchor
	edge_5: like edge_anchor
	edge_6: like edge_anchor
	edge_7: like edge_anchor
	edge_8: like edge_anchor
	edge_9: like edge_anchor
	edge_10: like edge_anchor
	edge_11: like edge_anchor
	edge_12: like edge_anchor
	edge_13: like edge_anchor
	edge_14: like edge_anchor

	graphs: HASH_TABLE [STRING_8, like graph_anchor]
			-- Pairs a string identifier to a graph

	nodes: HASH_TABLE [STRING_8, like node_anchor]
			-- Pairs a string identifier to a node

	edges: HASH_TABLE [STRING_8, like edge_anchor]
			-- Pairs a string identifier to an edge

feature -- Status report

	is_connected
			-- Test {EDGE}.`is_connected'
		local
			fn: STRING_8
			e: like edge_anchor
		do
			fn := "is_connected"
			divider (fn)
			from edges.start
			until edges.after
			loop
				e := edges.key_for_iteration
				call_routine (agent e.is_connected, [], fn, false)
				edges.forth
			end
		end

feature -- Basic operations

	connect
			-- Test {EDGE}.`connect', {EDGE}.`is_connected', {NODE}.`node_from',
			-- and {NONE}.`node_to
		local
			fn: STRING_8
		do
			fn := "connect"
			divider (fn)
			call_routine (agent edge_1.connect, [node_a, node_b], fn, 0)
			call_routine (agent edge_1.is_connected, [], "is_connected", true)
			call_routine (agent edge_1.node_from, [], "node_from", node_a)
			call_routine (agent edge_1.node_to, [], "node_to", node_b)
			call_routine (agent edge_1.is_directed, [], "is_directed", false)
--			call_routine (agent edge_2.connect, [node_a, node_c], "connect", 0)
--			call_routine (agent edge_2.set_directed, [], "set_directed", 0)
--			call_routine (agent edge_2.is_directed, [], "is_directed", true)
		end

feature -- Implementation

	call_routine (a_routine: ROUTINE [ANY, detachable TUPLE]; a_args: TUPLE;
					a_name: STRING_8; a_expected: ANY)
			-- Execute `a_routine' using `a_args'.
			-- Output the `signature' of the call.
			-- If `a_routine' is a function, compare the result with `a_expected'.
		require
			target_closed: attached a_routine.target
--			arguments_closed: a_routine.open_count = 0
			other_arguments_closed: attached {ROUTINE} a_expected as r implies
										r.open_count = 0
		local
			s: STRING_8
			ans: ANY
			is_ok: BOOLEAN
			ops: like {PROCEDURE}.operands
		do
			check attached a_routine.target as t then
				s := signature (t, a_name, a_args)
					-- Execute `a_routine_or_value' or use `out'
				if attached {PROCEDURE} a_routine as p then
					ops := p.operands
					p.call (a_args)
					ans := ""
					is_ok := true
				elseif attached {FUNCTION [ANY, TUPLE, ANY]} a_routine as f then
					ans := f.item (a_args)
					is_ok := ans.out ~ a_expected.out
					s := s + " = "
				else
					check
						should_not_happen: False
							-- because `a_routine' is a {PROCEDURE} or a {FUNCTION}
					end
					ans := " should_not_happen "
				end
				io.put_string (s + as_named (ans) + "%N")
				if not is_ok then
					io.put_string ("%T  ERROR -- expected  " + as_named (a_expected) + "%N")
				end
				assert (s, is_ok)
			end
		end

	signature (a_target: ANY; a_feature: STRING_8; a_arguments: TUPLE): STRING_8
			-- Create a string representing a feature's signature.
		local
			i: INTEGER
			a: detachable ANY
		do
			Result := a_target.generating_type + ":  "
			Result := Result + "(" + as_named (a_target) + ")"
			Result := Result + "." + a_feature
			if a_arguments.count > 0 then
					-- Add agruments
				Result := Result + " ("
				from i := 1
				until i > a_arguments.count
				loop
					a := a_arguments[i]
					Result := Result + as_named (a)
					if i < a_arguments.count then
						Result := Result + ", "
					end
					i := i + 1
				end
				Result := Result + ")"
			end
		end

	as_named (a_any: detachable ANY): STRING_8
		do
			Result := ""
			if attached {STRING} a_any as s then
				Result := Result + "%"" + s.out + "%""
			elseif attached {like graph_anchor} a_any as g then
				Result := Result + as_named_graph (g)
			elseif attached {like node_anchor} a_any as n then
				Result := Result + as_named_node (n)
			elseif attached {like edge_anchor} a_any as e then
				Result := Result + as_named_edge (e)
			elseif attached a_any then
				Result := Result + a_any.out
			else
				Result := Result + "Void"
			end
		end

	as_named_graph (a_graph: like graph_anchor): STRING_8
			-- A human readable identification of `a_graph'
		do
			check attached graphs.item (a_graph) as s then
				Result := s
			end
		end

	as_named_node (a_node: like node_anchor): STRING_8
			-- A human readable idetifacation of `a_node'
		do
			check attached nodes.item (a_node) as s then
				Result := s
			end
		end

	as_named_edge (a_edge: like edge_anchor): STRING_8
			-- A human readable idetifacation of `a_edge'
		do
			check attached edges.item (a_edge) as s then
				Result := s
			end
		end

	divider (a_string: STRING_8)
			-- Print a dividing line containing `a_string'
			-- (e.g.  "----------- a_string ------------"
		local
			w, c, n, i: INTEGER_32
		do
			io.put_string ("%N%N%N")
			w := 70
			c := a_string.count
			n := (w - c) // 2
			from i := 1
			until i > n
			loop
				io.put_string ("-")
				i := i + 1
			end
			io.put_string (" " + a_string + " ")
			from i := 1
			until i > n
			loop
				io.put_string ("-")
				i := i + 1
			end
			io.put_string ("%N")
		end

feature {NONE} -- Implementation

	graph_anchor: GRAPH
			-- Anchor for features using a graph.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			not_callable: False
		do
			check
				do_not_call: False then
					-- Because give no info; simply used as anchor.
			end
		end

	node_anchor: NODE
			-- Anchor for features using nodes.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			not_callable: False
		do
			check
				do_not_call: False then
					-- Because give no info; simply used as anchor.
			end
		end

	edge_anchor: EDGE
			-- Anchor for features using nodes.
			-- Not to be called; just used to anchor types.
			-- Declared as a feature to avoid adding an attribute.
		require
			not_callable: False
		do
			check
				do_not_call: False then
					-- Because give no info; simply used as anchor.
			end
		end

end
